#include "postgres.h"

#include "catalog/pg_type.h"
#include "libpq/pqformat.h"
#include "utils/array.h"
#include "utils/fmgroids.h"
#include "utils/memutils.h"
#include "utils/varbit.h"

/* bitor
 * perform a logical OR on two bit strings.
 */
Datum
bitor2(PG_FUNCTION_ARGS)
{
    VarBit     *arg1 = PG_GETARG_VARBIT_P(0);
    VarBit     *arg2 = PG_GETARG_VARBIT_P(1);
    VarBit     *result;
    int         len,
                bitlen1,
                bitlen2,
                i;
    bits8      *p1,
               *p2,
               *r;
    bits8       mask;

    bitlen1 = VARBITLEN(arg1);
    bitlen2 = VARBITLEN(arg2);

    // make sure arg1 is the larger of the two
    if (bitlen1 > bitlen2) {
      VarBit *tmp = arg1;
      int tmplen = bitlen1;
      arg1 = arg2;
      arg2 = tmp;
      bitlen1 = bitlen2;
      bitlen2 = tmplen;
    }

    len = VARSIZE(arg1);
    result = (VarBit *) palloc(len);
    VARATT_SIZEP(result) = len;
    VARBITLEN(result) = bitlen1;

    p1 = VARBITS(arg1);
    p2 = VARBITS(arg2);
    r = VARBITS(result);
    for (i = 0; i < VARBITBYTES(arg2); i++)
        *r++ = *p1++ | *p2++;
    for (; i < VARBITBYTES(arg1); i++)
        *r++ = *p1;
    
    /* Pad the result */
    mask = BITMASK << VARBITPAD(result);
    if (mask)
    {
        r--;
        *r &= mask;
    }

    PG_RETURN_VARBIT_P(result);
}
